#!/bin/bash
# Filename: build-python.sh
# Automates the packaging process of the software "python" for
# use with Zenwalk Linux.  
# Copyright (C) 2009 Michael Verret michael.verret@gmail.com
# Software source: http://python.sourceforge.net/
# Software dependencies: "bzip2 db44 expat fontconfig freetype gdbm ncurses \
#	openssl python readline sqlite tcl tk xcb xorg-libs zlib"

WorkArea=$(pwd)
SoftwareOrigName='Python'
SoftwareName='python3'
SoftwareVersion='3.0.1'
PythonVersion='3.0'
CompressionAlgorithm='tar.bz2'
Arch='i686'
Build='62.2'
Package="$SoftwareName-$SoftwareVersion-$Arch-$Build"
InstallDir="$WorkArea/$Package"
DocDir="$InstallDir/usr/doc/$SoftwareName-$SoftwareVersion"
SrcDir="$InstallDir/usr/src/$SoftwareName-$SoftwareVersion"
Documentation="AUTHORS COPYING ChangeLog INSTALL NEWS README TODO"

# Files from previous builds are considered cruft:
Cruft="$SoftwareName-$SoftwareVersion $Package $Package.*"
for Each in ${Cruft} ; do
	if [ -e $Each ] ; then
		rm -rf $Each
	fi
done

tar -xvjf $SoftwareOrigName-$SoftwareVersion.$CompressionAlgorithm
mv $SoftwareOrigName-$SoftwareVersion $SoftwareName-$SoftwareVersion

cd $SoftwareName-$SoftwareVersion
	chown -R root:root .
	find . \
	\( -perm 777 -o -perm 775 -o -perm 711 -o -perm 555 -o -perm 511 \) \
	-exec chmod 755 {} \; -o \
	\( -perm 666 -o -perm 664 -o -perm 600 -o -perm 444 -o -perm 440 -o -perm 400 \) \
	-exec chmod 644 {} \;

./configure CFLAGS="-O2 -march=$Arch -pipe" CXXFLAGS="-O2 -march=$Arch -pipe" \
	--prefix=/usr --libdir=/usr/lib --mandir=/usr/man \
    --docdir=/usr/doc/$SoftwareName-$SoftwareVersion --enable-ipv6 \
	--with-fpectl --enable-shared

make DESTDIR=$InstallDir || exit 1

# Replace rest of /usr/local/bin/python or /usr/bin/python3.x with
# /usr/bin/python3:
find . -wholename "./Parser" -prune -o -name '*.py' -type f -print0 | \
xargs -0 grep -lE '^#! *(/usr/.*bin/(env +)?)?python' | xargs sed -r -i -e \
'1s@^#![[:space:]]*(/usr/(local/)?bin/(env +)?)?python([0-9]+\.[0-9]+)?@#!/usr/bin/python3@'

make OPT="-O3 -march=$Arch -pipe -fPIC" DESTDIR=$InstallDir install || exit 1

# Remove duplicates:
fdupes $InstallDir/usr/lib/python$PythonVersion

# Install the demos:
find Demo/ -type d \( -regex ".*/.cvsignore" \) -exec rm -f \{\} \;
for x in `find Demo/ \( -not -name Makefile \) -print | sort` ; do
  test -d $x && ( install -c -m 755 -d $InstallDir/usr/doc/$SoftwareName-$SoftwareVersion/$x ) \
             || ( install -c -m 644 $x $InstallDir/usr/doc/$SoftwareName-$SoftwareVersion/$x )
done

# Install site-packages:
(
	cd $WorkArea/$SoftwareName-$SoftwareVersion/Tools ;
	mkdir -p $InstallDir/usr/lib/python$PythonVersion/site-packages || exit 1
	cp -aR * $InstallDir/usr/lib/python$PythonVersion/site-packages/ || exit 1
)
(
	cd $InstallDir/usr/doc/$SoftwareName-$SoftwareVersion/ ;
	ln -s /usr/lib/python$PythonVersion/site-packages/README \
		README.python-tools || exit 1
	ln -s /usr/lib/python$PythonVersion/site-packages/ Tools || exit 1
)

# A symlink for convenience:
(
	cd $InstallDir/usr/bin/
	ln -s python$PythonVersion python3
)
# Rename idle to prevent overwriting the 2x version:
(
	cd $InstallDir/usr/bin/
	mv idle idle$PythonVersion
	ln -s idle$PythonVersion idle3
)
# Rename pydoc to prevent overwriting the 2x version:
(
	cd $InstallDir/usr/bin/
	mv pydoc pydoc$PythonVersion
	ln -s pydoc$PythonVersion pydoc3
)
# Rename smtpd.py to prevent overwriting the 2x version:
(
	cd $InstallDir/usr/bin/
	mv smtpd.py smtpd$PythonVersion.py
	ln -s smtpd$PythonVersion.py smtpd3.py
)


mv $InstallDir/usr/bin/pydoc $InstallDir/usr/bin/pydoc3
rm -f $InstallDir/usr/bin/smtpd.py



# Move idle config into /etc:
mkdir -p $InstallDir/etc/idle3
( 
    cd $InstallDir/usr/lib/python$PythonVersion/idlelib/
	mv config-extensions.def $InstallDir/etc/idle3/config-extensions.def.new
	mv config-highlight.def $InstallDir/etc/idle3/config-highlight.def.new
	mv config-keys.def $InstallDir/etc/idle3/config-keys.def.new
	mv config-main.def $InstallDir/etc/idle3/config-main.def.new
	ln -sf /etc/idle3/config-extensions.def config-extensions.def
	ln -sf /etc/idle3/config-highlight.def config-highlight.def
	ln -sf /etc/idle3/config-keys.def config-keys.def
	ln -sf /etc/idle3/config-main.def config-main.def
)
mkdir -p $InstallDir/install
zcat $WorkArea/doinst-python.sh.gz > $InstallDir/install/doinst.sh

# Complete the remaining packaging actions:
cd $InstallDir
	find . | xargs file | grep "executable" | grep ELF | cut -f 1 -d : | \
		xargs strip --strip-unneeded 2> /dev/null
	find . | xargs file | grep "shared object" | grep ELF | cut -f 1 -d : | \
		xargs strip --strip-unneeded 2> /dev/null
	gzip -9 usr/man/*/*

mkdir -p $InstallDir/install
cat <<EOF0 > $InstallDir/install/slack-desc 
       |-----handy-ruler------------------------------------------------------|
python3: Object-oriented interpreted programming language
python3:
python3: Python 3 (a.k.a. "Python 3000" or "Py3k") is a new version of the
python3: language that is incompatible with the 2.x line of releases. The
python3: language is mostly the same, but many details, especially how built-
python3: in objects like dictionaries and strings work, have changed
python3: considerably, and a lot of deprecated features have finally been
python3: removed. Also, the standard library has been reorganized in a few
python3: prominent places.
python3:
python3: It is safe to install this on a system which already has 2.x version.
EOF0

# Remove zero length files:
rm $InstallDir/usr/lib/python3.0/site-packages/modulator/Templates/copyright
rm $InstallDir/usr/lib/python3.0/site-packages/pybench/package/__init__.py
rm $InstallDir/usr/lib/python3.0/site-packages/pybench/package/submodule.py
rm $InstallDir/usr/lib/python3.0/test/nullcert.pem
rm $InstallDir/usr/lib/python3.0/email/mime/__init__.py
rm $InstallDir/usr/lib/python3.0/email/test/__init__.py
rm $InstallDir/usr/lib/python3.0/sqlite3/test/__init__.py
rm $InstallDir/usr/lib/python3.0/urllib/__init__.py
rm $InstallDir/usr/lib/python3.0/build_class.py

cat <<EOF1 > $InstallDir/install/doinst.sh
touch usr/lib/python3.0/site-packages/modulator/Templates/copyright
touch usr/lib/python3.0/site-packages/pybench/package/__init__.py
touch usr/lib/python3.0/site-packages/pybench/package/submodule.py
touch usr/lib/python3.0/test/nullcert.pem
touch usr/lib/python3.0/email/mime/__init__.py
touch usr/lib/python3.0/email/test/__init__.py
touch usr/lib/python3.0/sqlite3/test/__init__.py
touch usr/lib/python3.0/urllib/__init__.py
touch usr/lib/python3.0/build_class.py
EOF1


mkdir -p $DocDir
cd $WorkArea/$SoftwareName-$SoftwareVersion
	cp $Documentation $DocDir

cd $DocDir ; find . -name "*" -size 0c -exec rm {} \;

mkdir -p $SrcDir
cd $WorkArea ; cp build-$SoftwareName.sh $WorkArea/doinst-python.sh.gz $SrcDir

cd $InstallDir ; makepkg -l y -c n $InstallDir.tlz
	
cd $WorkArea
	ADD="python" depfinder -f $Package.tlz
	md5sum $Package.tlz > $Package.md5
	rm -rf $SoftwareName-$SoftwareVersion $InstallDir

# CHANGELOG:
# ~ Mon Apr 20 2009	Michael Verret "axxium" michael.verret@gmail.com
# ~ Updated: python 3.0.1
